package kck.logic;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.*;

/** Dane HTD
 * Klasa obsluguje pobranie danych i przechowywanie ich
 * 
 * pakiety w pliku co 10ms
 * rozpoznawanie okregi, romby, prostokaty, strzalki
 * strzalki w 8 kierunkach
 * figury robione wieloma pociagnieciami
 * pressure: 0-1000
 *
 */
public class HTDData {
	public static int maxPressure = 1024;
	
	protected long maxWidth;
	protected long maxHeight;
	
	protected String filename;
	protected String date;
	protected String memo;
	protected int minx;
	protected int maxx;
	protected int miny;
	protected int maxy;
	protected int pointsCount;
	
	protected double xScalingRatio;
	protected double yScalingRatio;
	
	protected int time[];
	public int x[];
	public int y[];
	protected int pressure[];
	protected int azimuth[];
	protected int altitude[];
	
	protected double maxSeqDist;
	private int lastDrawn;
	
	protected BufferedImage dataDraw;
	public ColorScale cs;
	
	public void initDraw() {
		int i;
		Graphics g;
		
		if (pointsCount > 0) {
			
			dataDraw = new BufferedImage((int)(getMaxx() - getMinx()), (int)(getMaxy() - getMinx()), BufferedImage.TYPE_INT_ARGB);
			g = dataDraw.getGraphics();
			
			for (i=0; i<getPointsCount(); i++) {
				drawPoint(g, i);
			}
		}
	}
	
	public void timedDrawInit() {
		if (pointsCount > 0) {
			dataDraw = new BufferedImage((int)(getMaxx() - getMinx()), (int)(getMaxy() - getMinx()), BufferedImage.TYPE_INT_ARGB);
		}		
		lastDrawn = -1;
	}
	
	public boolean timedDraw(int timestamp) {
		do {
			lastDrawn++;
			drawPoint(dataDraw.getGraphics(), lastDrawn);
		} while ((lastDrawn < pointsCount-1) && (time[lastDrawn] < timestamp));
		
		return lastDrawn < pointsCount-1;
	}
	
	protected void drawPoint(Graphics g, int no) {
		g.setColor(cs.getColor(no));
		g.drawOval(x[no], y[no], 2, 2);
	}
	
	
	public HTDData() {
		maxWidth = 1000;
		maxHeight = 1000;
		maxSeqDist = 999999;
		
		cs = new ColorScale();
		cs.setData(this);
	}
	
	public HTDData(HTDData source) {
		this();
		copyHeader(source);
		copyBody(source);
	}

	protected void copyHeader(HTDData source) {
		filename = new String(source.filename);
		date = new String(source.date);
		memo = new String(source.memo);
		minx = source.minx;
		maxx = source.maxx;
		miny = source.miny;
		maxy = source.maxy;
		pointsCount = source.pointsCount;
		xScalingRatio = source.xScalingRatio;
		yScalingRatio = source.yScalingRatio;
	}
	
	protected void copyBody(HTDData source) {
		time = new int[pointsCount];
		x = new int[pointsCount];
		y = new int[pointsCount];
		pressure = new int[pointsCount];
		azimuth = new int[pointsCount];
		altitude = new int[pointsCount];
		
		System.arraycopy(source.time, 0, time, 0, pointsCount);
		System.arraycopy(source.x, 0, x, 0, pointsCount);
		System.arraycopy(source.y, 0, y, 0, pointsCount);
		System.arraycopy(source.pressure, 0, pressure, 0, pointsCount);
		System.arraycopy(source.azimuth, 0, azimuth, 0, pointsCount);
		System.arraycopy(source.altitude, 0, altitude, 0, pointsCount);
	}
		
	public String getDate() {
		return date;
	}

	public long getMiny() {
		return miny;
	}

	public long getMaxy() {
		return maxy;
	}

	public String getFilename() {
		return filename;
	}

	public String getMemo() {
		return memo;
	}

	public long getMinx() {
		return minx;
	}

	public long getMaxx() {
		return maxx;
	}

	public int getPointsCount() {
		return pointsCount;
	}
	
	private int readInt(FileInputStream f) throws IOException {
		byte buffer[] = new byte[4];
		f.read(buffer, 0, 4);
		return (buffer[0] & 0xFF) | (((int)buffer[1] & 0xFF) << 8) | (((int)buffer[2] & 0xFF) << 16) | (((int)buffer[3] & 0xFF) << 24);
	}
	
	private String readString(FileInputStream f, int len) throws IOException {
		byte buffer[] = new byte[len];

		f.read(buffer, 0, len);
		return new String(buffer);
	}
	
	private void loadHeader(FileInputStream f) throws IOException {
		filename = readString(f, readInt(f));
		date = readString(f, readInt(f));
		memo = readString(f, readInt(f));
		minx = readInt(f);
		maxx = readInt(f);
		miny = readInt(f);
		maxy = readInt(f);
		pointsCount = readInt(f);
		
		xScalingRatio = (maxx - minx > maxWidth) ? (double)maxWidth/(maxx-minx) : 1;
		yScalingRatio = (maxy - miny > maxHeight) ? (double)maxHeight/(maxy-miny) : 1;
		
		minx *= xScalingRatio;
		maxx *= xScalingRatio;
		miny *= yScalingRatio;
		maxy *= yScalingRatio;
	}
	
	private void loadBody(FileInputStream f) throws IOException {
		int l;
				
		for (l=0; l< pointsCount; l++) {
			time[l] = readInt(f);
			x[l] = (int)((xScalingRatio*readInt(f)) - minx);
			y[l] = (int)(maxy - (yScalingRatio*readInt(f)) - miny); // invert by X axis
			pressure[l] = readInt(f);
			azimuth[l] = readInt(f);
			altitude[l] = readInt(f);
			
			if (pressure[l] == 0) {
				pointsCount--;
				l--;
			}
			
			if ((l>0) && (time[l] - time[l-1] == 10) && (distance(l, l-1) > maxSeqDist)) {
				l += addVirtualPoints(l-1);
			}
		}
	}
	
	/**
	 * Wstaw sztuczne punkty midzy punktami from a to. Zwikszenie
	 * rozdzielczoci rejestrowanego obrazu. zwraca liczb dodanych punktw
	 * @param from
	 * @param to
	 */
	private int addVirtualPoints(int from) {
		int c = (int)Math.floor(distance(from, from+1)/maxSeqDist)+1;
		int i = 1;
		int dx = (x[from+1] - x[from]) / (c+1);
		int dy = (y[from+1] - y[from]) / (c+1);
		int daltitude = (altitude[from+1] - altitude[from]) / (c+1);
		int dazimuth = (azimuth[from+1] - azimuth[from]) / (c+1);
		int dpressure = (pressure[from+1] - pressure[from]) / (c+1);
		int dtime = (time[from+1] - time[from]) / (c+1);
		
		x[from+c+1] = x[from+1];
		y[from+c+1] = y[from+1];
		azimuth[from+c+1] = azimuth[from+1];
		pressure[from+c+1] = pressure[from+1];
		altitude[from+c+1] = altitude[from+1];
		time[from+c+1] = time[from+1];
		
		//System.out.println("add "+c+" points between "+x[from]+"x"+y[from]+" and "+x[from+1]+"x"+y[from+1]+" ("+from+"-"+(from+c+1)+")");
		while ((pointsCount < x.length) && (c >= i)) {
			x[from+i] = x[from] + dx*i;
			y[from+i] = y[from] + dy*i;
			altitude[from+i] = altitude[from] + daltitude*i;
			azimuth[from+i] = azimuth[from] + dazimuth*i;
			pressure[from+i] = pressure[from] + dpressure*i;
			time[from+i-1] = time[from] + dtime*i;
			
			//System.out.println("\t"+(from+i)+". "+x[from+i]+"x"+y[from+i-1]);
			
			pointsCount++;
			i++;
		}
		return c;
	}
	
	public void setMaxSeqDist(double d) {
		maxSeqDist = d;
	}
	
	/**
	 * odlego midzy punktem i-tym a j-tym
	 * @param i
	 * @param j
	 * @return
	 */
	public double distance(int i, int j) {
		return Math.sqrt((x[i]-x[j])*(x[i]-x[j]) + (y[i]-y[j])*(y[i]-y[j]));
	}
	
	public void load(File file) throws IOException {
		FileInputStream f = new FileInputStream(file);
		
		loadHeader(f);
		
		time = new int[pointsCount*4/3];
		x = new int[pointsCount*4/3];
		y = new int[pointsCount*4/3];
		pressure = new int[pointsCount*4/3];
		azimuth = new int[pointsCount*4/3];
		altitude = new int[pointsCount*4/3];
		loadBody(f);
		
		f.close();
	}
	
	public BufferedImage getDataDraw() {
		return dataDraw;
	}
}
